@@ -74,7 +74,7 @@ def login(dhurl, user, password, errors):
 def deploy_application(dhurl, cookies, appname, appversion, env):
     """Deploy the application to the environment
     Returns: deployment_id"""
-    data = get_application(dhurl, cookies, appname, appversion)
+    data = get_application(dhurl, cookies, appname, appversion, True)
     appid = data[0]
 
     data = get_json(dhurl + "/dmadminweb/API/deploy?app=" + str(appid) + "&env=" + urllib.parse.quote(env), cookies)
@@ -90,7 +90,7 @@ def deploy_application(dhurl, cookies, appname, appversion, env):
 
 def move_application(dhurl, cookies, appname, appversion, from_domain, task):
     """Move an application from the from_domain using the task"""
-    data = get_application(dhurl, cookies, appname, appversion)
+    data = get_application(dhurl, cookies, appname, appversion, True)
     appid = data[0]
 
     # Get from domainid
@@ -119,7 +119,7 @@ def move_application(dhurl, cookies, appname, appversion, from_domain, task):
 
 def approve_application(dhurl, cookies, appname, appversion):
     """Approve the application for the current domain that it is in."""
-    data = get_application(dhurl, cookies, appname, appversion)
+    data = get_application(dhurl, cookies, appname, appversion, True)
     appid = data[0]
 
     data = get_json(dhurl + "/dmadminweb/API/approve/" + appid, cookies)
@@ -260,7 +260,7 @@ def clean_name(name):
     return name
 
 
-def get_component(dhurl, cookies, compname, compvariant, compversion):
+def get_component(dhurl, cookies, compname, compvariant, compversion, id_only):
     compvariant = clean_name(compvariant)
     compversion = clean_name(compversion)
 
@@ -290,7 +290,11 @@ def get_component(dhurl, cookies, compname, compvariant, compversion):
     else:
         check_compname = short_compname
 
-    data = get_json(dhurl + "/dmadminweb/API/component/?name=" + urllib.parse.quote(component), cookies)
+    param = ""
+    if (id_only):
+        param = "&idonly=Y"
+
+    data = get_json(dhurl + "/dmadminweb/API/component/?name=" + urllib.parse.quote(component) + param, cookies)
 
     if (data is None):
         return [-1, ""]
@@ -314,7 +318,7 @@ def get_component(dhurl, cookies, compname, compvariant, compversion):
 
 def get_component_name(dhurl, cookies, compid):
     name = ""
-    data = get_json(dhurl + "/dmadminweb/API/component/" + str(compid), cookies)
+    data = get_json(dhurl + "/dmadminweb/API/component/" + str(compid) + "?idonly=Y", cookies)
 
     if (data is None):
         return name
@@ -324,9 +328,14 @@ def get_component_name(dhurl, cookies, compid):
     return name
 
 
-def get_application_name(dhurl, cookies, appid):
+def get_application_name(dhurl, cookies, appid, id_only):
     name = ""
-    data = get_json(dhurl + "/dmadminweb/API/application/" + str(appid), cookies)
+
+    param = ""
+    if (id_only):
+        param = "&idonly=Y"
+
+    data = get_json(dhurl + "/dmadminweb/API/application/" + str(appid) + param, cookies)
 
     if (data is None):
         return name
@@ -345,11 +354,11 @@ def new_component_version(dhurl, cookies, compname, compvariant, compversion, ki
         compversion = None
 
     # Get latest version of compnent variant
-    data = get_component(dhurl, cookies, compname, compvariant, compversion)
+    data = get_component(dhurl, cookies, compname, compvariant, compversion, False)
     if (data[0] == -1):
-        data = get_component(dhurl, cookies, compname, compvariant, None)
+        data = get_component(dhurl, cookies, compname, compvariant, None, False)
         if (data[0] == -1):
-            data = get_component(dhurl, cookies, compname, "", None)
+            data = get_component(dhurl, cookies, compname, "", None, False)
 
     compid = data[0]
     found_compname = data[1]
@@ -519,7 +528,7 @@ def new_component(dhurl, cookies, compname, compvariant, compversion, kind, pare
 
 def update_component_attrs(dhurl, cookies, compname, compvariant, compversion, attrs):
     # Get latest version of compnent variant
-    data = get_component(dhurl, cookies, compname, compvariant, compversion)
+    data = get_component(dhurl, cookies, compname, compvariant, compversion, True)
     compid = data[0]
 
     if (compid < 0):
@@ -550,7 +559,7 @@ def update_component_attrs(dhurl, cookies, compname, compvariant, compversion, a
     return [False, "No attributes to update on '" + compname + "'"]
 
 
-def get_application(dhurl, cookies, appname, appversion):
+def get_application(dhurl, cookies, appname, appversion, id_only):
     appversion = clean_name(appversion)
 
     application = ""
@@ -560,7 +569,11 @@ def get_application(dhurl, cookies, appname, appversion):
     else:
         application = appname
 
-    data = get_json(dhurl + "/dmadminweb/API/application/?name=" + urllib.parse.quote(application), cookies)
+    param = ""
+    if (id_only):
+        param = "&idonly=Y"
+
+    data = get_json(dhurl + "/dmadminweb/API/application/?name=" + urllib.parse.quote(application) + param, cookies)
 
     if (data is None):
         return [-1, "", -1]
@@ -581,21 +594,17 @@ def get_application(dhurl, cookies, appname, appversion):
     return [-1, "", -1]
 
 
-def get_base_component(dhurl, cookies, compid):
-    data = get_json(dhurl + "/dmadminweb/API/component/" + str(compid), cookies)
+def get_base_component(dhurl, cookies, compid, id_only):
+
+    data = get_json(dhurl + "/dmadminweb/API/basecomponent/" + str(compid), cookies)
 
     if (data is None):
         return [-1, "", -1]
 
     result = data.get('result', None)
 
-    while (result and result.get('predecessor', None)):
-        data2 = get_json(dhurl + "/dmadminweb/API/component/" + str(result['predecessor']['id']), cookies)
-
-        if (data2 is None):
-            break
-
-        result = data2.get('result', None)
+    if (result is None):
+        return [-1, "", -1]
 
     return result['id']
 
@@ -616,14 +625,14 @@ def new_application(dhurl, cookies, appname, appversion, envs):
         appname = appname.split('.')[-1]
 
     # Get Base Version
-    data = get_application(dhurl, cookies, appname, "")
+    data = get_application(dhurl, cookies, appname, "", True)
     parent_appid = data[0]
 
     # Create base version
     if (parent_appid < 0):
         data = get_json(dhurl + "/dmadminweb/API/new/application/?name=" + urllib.parse.quote(appname) + "&" + domain, cookies)
         if (data.get('success', False)):
-            data = get_application(dhurl, cookies, appname, "")
+            data = get_application(dhurl, cookies, appname, "", True)
             parent_appid = data[0]
 
         if (envs is not None):
@@ -631,11 +640,11 @@ def new_application(dhurl, cookies, appname, appversion, envs):
                 data = get_json(dhurl + "/dmadminweb/API/assign/application/?name=" + urllib.parse.quote(appname) + "&env=" + urllib.parse.quote(env), cookies)
 
     # Refetch parent to get version list
-    data = get_application(dhurl, cookies, appname, "")
+    data = get_application(dhurl, cookies, appname, "", True)
     latest_appid = data[2]
 
     # Refetch the current app version to see if we need to create it or not
-    data = get_application(dhurl, cookies, appname, appversion)
+    data = get_application(dhurl, cookies, appname, appversion, True)
     appid = data[0]
 
     if (appid < 0):
@@ -651,7 +660,7 @@ def new_application(dhurl, cookies, appname, appversion, envs):
 
 def add_compver_to_appver(dhurl, cookies, appid, compid):
     replace_compid = -1
-    basecompid = get_base_component(dhurl, cookies, compid)
+    basecompid = get_base_component(dhurl, cookies, compid, True)
     lastcompid = 0
     xpos = 100
     ypos = 100
@@ -669,7 +678,7 @@ def add_compver_to_appver(dhurl, cookies, appid, compid):
             lastcompid = result.get('lastcompver', -1)
 
             for comp in complist:
-                app_basecompid = get_base_component(dhurl, cookies, comp['id'])
+                app_basecompid = get_base_component(dhurl, cookies, comp['id'], True)
                 if (app_basecompid == basecompid):
                     replace_compid = comp['id']
 
@@ -689,6 +698,18 @@ def assign_comp_to_app(dhurl, cookies, appid, compid, parent_compid, xpos, ypos)
     get_json(dhurl + "/dmadminweb/UpdateAttrs?f=acvm&a=" + str(appid) + "&c=" + str(compid) + "&xpos=" + str(xpos) + "&ypos=" + str(ypos), cookies)
     get_json(dhurl + "/dmadminweb/UpdateAttrs?f=cal&a=" + str(appid) + "&fn=" + str(parent_compid) + "&tn=" + str(compid), cookies)
 
+def assign_app_to_env(dhurl, cookies, appname, envs):
+    domain = ""
+    if ('.' in appname):
+        parts = appname.split('.')
+        if (parts):
+            parts.pop()
+        domain = '.'.join(parts)
+        domain = "domain=" + urllib.parse.quote(domain)
+        appname = appname.split('.')[-1]
+    if (envs is not None):
+        for env in envs:
+            get_json(dhurl + "/dmadminweb/API/assign/application/?name=" + urllib.parse.quote(appname) + "&env=" + urllib.parse.quote(env), cookies)
 
 def clone_repo(project):
     print("### Grabbing features.toml ###")
